/* Copyright © 2019 Raheman Vaiya.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _KEYS_H_
#define _KEYS_H_
#define _KEYS_H_

#include <linux/input-event-codes.h>
#include <stdint.h>

#define ISMOD(keycode) \
    (((keycode) == KEY_LEFTCTRL) ||\
     ((keycode) == KEY_LEFTALT) ||\
     ((keycode) == KEY_LEFTSHIFT) ||\
     ((keycode) == KEY_LEFTMETA) ||\
     ((keycode) == KEY_RIGHTCTRL) ||\
     ((keycode) == KEY_RIGHTALT) ||\
     ((keycode) == KEY_RIGHTSHIFT) ||\
     ((keycode) == KEY_RIGHTMETA))

#define MOD_ALT_GR 0x10
#define MOD_CTRL 0x8
#define MOD_SHIFT 0x4
#define MOD_SUPER 0x2
#define MOD_ALT 0x1

static const char *keycode_strings[] = {
	[KEY_ESC] = "esc",
	[KEY_1] = "1",
	[KEY_2] = "2",
	[KEY_3] = "3",
	[KEY_4] = "4",
	[KEY_5] = "5",
	[KEY_6] = "6",
	[KEY_7] = "7",
	[KEY_8] = "8",
	[KEY_9] = "9",
	[KEY_0] = "0",
	[KEY_MINUS] = "minus",
	[KEY_EQUAL] = "equal",
	[KEY_BACKSPACE] = "backspace",
	[KEY_TAB] = "tab",
	[KEY_Q] = "q",
	[KEY_W] = "w",
	[KEY_E] = "e",
	[KEY_R] = "r",
	[KEY_T] = "t",
	[KEY_Y] = "y",
	[KEY_U] = "u",
	[KEY_I] = "i",
	[KEY_O] = "o",
	[KEY_P] = "p",
	[KEY_LEFTBRACE] = "leftbrace",
	[KEY_RIGHTBRACE] = "rightbrace",
	[KEY_ENTER] = "enter",
	[KEY_LEFTCTRL] = "leftctrl",
	[KEY_A] = "a",
	[KEY_S] = "s",
	[KEY_D] = "d",
	[KEY_F] = "f",
	[KEY_G] = "g",
	[KEY_H] = "h",
	[KEY_J] = "j",
	[KEY_K] = "k",
	[KEY_L] = "l",
	[KEY_SEMICOLON] = "semicolon",
	[KEY_APOSTROPHE] = "apostrophe",
	[KEY_GRAVE] = "grave",
	[KEY_LEFTSHIFT] = "leftshift",
	[KEY_BACKSLASH] = "backslash",
	[KEY_Z] = "z",
	[KEY_X] = "x",
	[KEY_C] = "c",
	[KEY_V] = "v",
	[KEY_B] = "b",
	[KEY_N] = "n",
	[KEY_M] = "m",
	[KEY_COMMA] = "comma",
	[KEY_DOT] = "dot",
	[KEY_SLASH] = "slash",
	[KEY_RIGHTSHIFT] = "rightshift",
	[KEY_KPASTERISK] = "kpasterisk",
	[KEY_LEFTALT] = "leftalt",
	[KEY_SPACE] = "space",
	[KEY_CAPSLOCK] = "capslock",
	[KEY_F1] = "f1",
	[KEY_F2] = "f2",
	[KEY_F3] = "f3",
	[KEY_F4] = "f4",
	[KEY_F5] = "f5",
	[KEY_F6] = "f6",
	[KEY_F7] = "f7",
	[KEY_F8] = "f8",
	[KEY_F9] = "f9",
	[KEY_F10] = "f10",
	[KEY_NUMLOCK] = "numlock",
	[KEY_SCROLLLOCK] = "scrolllock",
	[KEY_KP7] = "kp7",
	[KEY_KP8] = "kp8",
	[KEY_KP9] = "kp9",
	[KEY_KPMINUS] = "kpminus",
	[KEY_KP4] = "kp4",
	[KEY_KP5] = "kp5",
	[KEY_KP6] = "kp6",
	[KEY_KPPLUS] = "kpplus",
	[KEY_KP1] = "kp1",
	[KEY_KP2] = "kp2",
	[KEY_KP3] = "kp3",
	[KEY_KP0] = "kp0",
	[KEY_KPDOT] = "kpdot",
	[KEY_ZENKAKUHANKAKU] = "zenkakuhankaku",
	[KEY_102ND] = "102nd",
	[KEY_F11] = "f11",
	[KEY_F12] = "f12",
	[KEY_RO] = "ro",
	[KEY_KATAKANA] = "katakana",
	[KEY_HIRAGANA] = "hiragana",
	[KEY_HENKAN] = "henkan",
	[KEY_KATAKANAHIRAGANA] = "katakanahiragana",
	[KEY_MUHENKAN] = "muhenkan",
	[KEY_KPJPCOMMA] = "kpjpcomma",
	[KEY_KPENTER] = "kpenter",
	[KEY_RIGHTCTRL] = "rightctrl",
	[KEY_KPSLASH] = "kpslash",
	[KEY_SYSRQ] = "sysrq",
	[KEY_RIGHTALT] = "rightalt",
	[KEY_LINEFEED] = "linefeed",
	[KEY_HOME] = "home",
	[KEY_UP] = "up",
	[KEY_PAGEUP] = "pageup",
	[KEY_LEFT] = "left",
	[KEY_RIGHT] = "right",
	[KEY_END] = "end",
	[KEY_DOWN] = "down",
	[KEY_PAGEDOWN] = "pagedown",
	[KEY_INSERT] = "insert",
	[KEY_DELETE] = "delete",
	[KEY_MACRO] = "macro",
	[KEY_MUTE] = "mute",
	[KEY_VOLUMEDOWN] = "volumedown",
	[KEY_VOLUMEUP] = "volumeup",
	[KEY_POWER] = "power",
	[KEY_KPEQUAL] = "kpequal",
	[KEY_KPPLUSMINUS] = "kpplusminus",
	[KEY_PAUSE] = "pause",
	[KEY_SCALE] = "scale",
	[KEY_KPCOMMA] = "kpcomma",
	[KEY_HANGEUL] = "hangeul",
	[KEY_HANJA] = "hanja",
	[KEY_YEN] = "yen",
	[KEY_LEFTMETA] = "leftmeta",
	[KEY_RIGHTMETA] = "rightmeta",
	[KEY_COMPOSE] = "compose",
	[KEY_STOP] = "stop",
	[KEY_AGAIN] = "again",
	[KEY_PROPS] = "props",
	[KEY_UNDO] = "undo",
	[KEY_FRONT] = "front",
	[KEY_COPY] = "copy",
	[KEY_OPEN] = "open",
	[KEY_PASTE] = "paste",
	[KEY_FIND] = "find",
	[KEY_CUT] = "cut",
	[KEY_HELP] = "help",
	[KEY_MENU] = "menu",
	[KEY_CALC] = "calc",
	[KEY_SETUP] = "setup",
	[KEY_SLEEP] = "sleep",
	[KEY_WAKEUP] = "wakeup",
	[KEY_FILE] = "file",
	[KEY_SENDFILE] = "sendfile",
	[KEY_DELETEFILE] = "deletefile",
	[KEY_XFER] = "xfer",
	[KEY_PROG1] = "prog1",
	[KEY_PROG2] = "prog2",
	[KEY_WWW] = "www",
	[KEY_MSDOS] = "msdos",
	[KEY_COFFEE] = "coffee",
	[KEY_ROTATE_DISPLAY] = "display",
	[KEY_CYCLEWINDOWS] = "cyclewindows",
	[KEY_MAIL] = "mail",
	[KEY_BOOKMARKS] = "bookmarks",
	[KEY_COMPUTER] = "computer",
	[KEY_BACK] = "back",
	[KEY_FORWARD] = "forward",
	[KEY_CLOSECD] = "closecd",
	[KEY_EJECTCD] = "ejectcd",
	[KEY_EJECTCLOSECD] = "ejectclosecd",
	[KEY_NEXTSONG] = "nextsong",
	[KEY_PLAYPAUSE] = "playpause",
	[KEY_PREVIOUSSONG] = "previoussong",
	[KEY_STOPCD] = "stopcd",
	[KEY_RECORD] = "record",
	[KEY_REWIND] = "rewind",
	[KEY_PHONE] = "phone",
	[KEY_ISO] = "iso",
	[KEY_CONFIG] = "config",
	[KEY_HOMEPAGE] = "homepage",
	[KEY_REFRESH] = "refresh",
	[KEY_EXIT] = "exit",
	[KEY_MOVE] = "move",
	[KEY_EDIT] = "edit",
	[KEY_SCROLLUP] = "scrollup",
	[KEY_SCROLLDOWN] = "scrolldown",
	[KEY_KPLEFTPAREN] = "kpleftparen",
	[KEY_KPRIGHTPAREN] = "kprightparen",
	[KEY_NEW] = "new",
	[KEY_REDO] = "redo",
	[KEY_F13] = "f13",
	[KEY_F14] = "f14",
	[KEY_F15] = "f15",
	[KEY_F16] = "f16",
	[KEY_F17] = "f17",
	[KEY_F18] = "f18",
	[KEY_F19] = "f19",
	[KEY_F20] = "f20",
	[KEY_F21] = "f21",
	[KEY_F22] = "f22",
	[KEY_F23] = "f23",
	[KEY_F24] = "f24",
	[KEY_PLAYCD] = "playcd",
	[KEY_PAUSECD] = "pausecd",
	[KEY_PROG3] = "prog3",
	[KEY_PROG4] = "prog4",
	[KEY_DASHBOARD] = "dashboard",
	[KEY_SUSPEND] = "suspend",
	[KEY_CLOSE] = "close",
	[KEY_PLAY] = "play",
	[KEY_FASTFORWARD] = "fastforward",
	[KEY_BASSBOOST] = "bassboost",
	[KEY_PRINT] = "print",
	[KEY_HP] = "hp",
	[KEY_CAMERA] = "camera",
	[KEY_SOUND] = "sound",
	[KEY_QUESTION] = "question",
	[KEY_EMAIL] = "email",
	[KEY_CHAT] = "chat",
	[KEY_SEARCH] = "search",
	[KEY_CONNECT] = "connect",
	[KEY_FINANCE] = "finance",
	[KEY_SPORT] = "sport",
	[KEY_SHOP] = "shop",
	[KEY_ALTERASE] = "alterase",
	[KEY_CANCEL] = "cancel",
	[KEY_BRIGHTNESSDOWN] = "brightnessdown",
	[KEY_BRIGHTNESSUP] = "brightnessup",
	[KEY_MEDIA] = "media",
	[KEY_SWITCHVIDEOMODE] = "switchvideomode",
	[KEY_KBDILLUMTOGGLE] = "kbdillumtoggle",
	[KEY_KBDILLUMDOWN] = "kbdillumdown",
	[KEY_KBDILLUMUP] = "kbdillumup",
	[KEY_SEND] = "send",
	[KEY_REPLY] = "reply",
	[KEY_FORWARDMAIL] = "forwardmail",
	[KEY_SAVE] = "save",
	[KEY_DOCUMENTS] = "documents",
	[KEY_BATTERY] = "battery",
	[KEY_BLUETOOTH] = "bluetooth",
	[KEY_WLAN] = "wlan",
	[KEY_UWB] = "uwb",
	[KEY_UNKNOWN] = "unknown",
	[KEY_VIDEO_NEXT] = "next",
	[KEY_VIDEO_PREV] = "prev",
	[KEY_BRIGHTNESS_CYCLE] = "cycle",
	[KEY_BRIGHTNESS_AUTO] = "auto",
	[KEY_DISPLAY_OFF] = "off",
	[KEY_WWAN] = "wwan",
	[KEY_RFKILL] = "rfkill",
	[KEY_MICMUTE] = "micmute",
	[KEY_OK] = "ok",
	[KEY_SELECT] = "select",
	[KEY_GOTO] = "goto",
	[KEY_CLEAR] = "clear",
	[KEY_POWER2] = "power2",
	[KEY_OPTION] = "option",
	[KEY_INFO] = "info",
	[KEY_TIME] = "time",
	[KEY_VENDOR] = "vendor",
	[KEY_ARCHIVE] = "archive",
	[KEY_PROGRAM] = "program",
	[KEY_CHANNEL] = "channel",
	[KEY_FAVORITES] = "favorites",
	[KEY_EPG] = "epg",
	[KEY_PVR] = "pvr",
	[KEY_MHP] = "mhp",
	[KEY_LANGUAGE] = "language",
	[KEY_TITLE] = "title",
	[KEY_SUBTITLE] = "subtitle",
	[KEY_ANGLE] = "angle",
	[KEY_ZOOM] = "zoom",
	[KEY_MODE] = "mode",
	[KEY_KEYBOARD] = "keyboard",
	[KEY_SCREEN] = "screen",
	[KEY_PC] = "pc",
	[KEY_TV] = "tv",
	[KEY_TV2] = "tv2",
	[KEY_VCR] = "vcr",
	[KEY_VCR2] = "vcr2",
	[KEY_SAT] = "sat",
	[KEY_SAT2] = "sat2",
	[KEY_CD] = "cd",
	[KEY_TAPE] = "tape",
	[KEY_RADIO] = "radio",
	[KEY_TUNER] = "tuner",
	[KEY_PLAYER] = "player",
	[KEY_TEXT] = "text",
	[KEY_DVD] = "dvd",
	[KEY_AUX] = "aux",
	[KEY_MP3] = "mp3",
	[KEY_AUDIO] = "audio",
	[KEY_VIDEO] = "video",
	[KEY_DIRECTORY] = "directory",
	[KEY_LIST] = "list",
	[KEY_MEMO] = "memo",
	[KEY_CALENDAR] = "calendar",
	[KEY_RED] = "red",
	[KEY_GREEN] = "green",
	[KEY_YELLOW] = "yellow",
	[KEY_BLUE] = "blue",
	[KEY_CHANNELUP] = "channelup",
	[KEY_CHANNELDOWN] = "channeldown",
	[KEY_FIRST] = "first",
	[KEY_LAST] = "last",
	[KEY_AB] = "ab",
	[KEY_NEXT] = "next",
	[KEY_RESTART] = "restart",
	[KEY_SLOW] = "slow",
	[KEY_SHUFFLE] = "shuffle",
	[KEY_BREAK] = "break",
	[KEY_PREVIOUS] = "previous",
	[KEY_DIGITS] = "digits",
	[KEY_TEEN] = "teen",
	[KEY_TWEN] = "twen",
	[KEY_VIDEOPHONE] = "videophone",
	[KEY_GAMES] = "games",
	[KEY_ZOOMIN] = "zoomin",
	[KEY_ZOOMOUT] = "zoomout",
	[KEY_ZOOMRESET] = "zoomreset",
	[KEY_WORDPROCESSOR] = "wordprocessor",
	[KEY_EDITOR] = "editor",
	[KEY_SPREADSHEET] = "spreadsheet",
	[KEY_GRAPHICSEDITOR] = "graphicseditor",
	[KEY_PRESENTATION] = "presentation",
	[KEY_DATABASE] = "database",
	[KEY_NEWS] = "news",
	[KEY_VOICEMAIL] = "voicemail",
	[KEY_ADDRESSBOOK] = "addressbook",
	[KEY_MESSENGER] = "messenger",
	[KEY_DISPLAYTOGGLE] = "displaytoggle",
	[KEY_SPELLCHECK] = "spellcheck",
	[KEY_LOGOFF] = "logoff",
	[KEY_DOLLAR] = "dollar",
	[KEY_EURO] = "euro",
	[KEY_FRAMEBACK] = "frameback",
	[KEY_FRAMEFORWARD] = "frameforward",
	[KEY_CONTEXT_MENU] = "menu",
	[KEY_MEDIA_REPEAT] = "repeat",
	[KEY_10CHANNELSUP] = "10channelsup",
	[KEY_10CHANNELSDOWN] = "10channelsdown",
	[KEY_IMAGES] = "images",
	[KEY_DEL_EOL] = "eol",
	[KEY_DEL_EOS] = "eos",
	[KEY_INS_LINE] = "line",
	[KEY_DEL_LINE] = "line",
	[KEY_FN] = "fn",
	[KEY_FN_ESC] = "esc",
	[KEY_FN_F1] = "f1",
	[KEY_FN_F2] = "f2",
	[KEY_FN_F3] = "f3",
	[KEY_FN_F4] = "f4",
	[KEY_FN_F5] = "f5",
	[KEY_FN_F6] = "f6",
	[KEY_FN_F7] = "f7",
	[KEY_FN_F8] = "f8",
	[KEY_FN_F9] = "f9",
	[KEY_FN_F10] = "f10",
	[KEY_FN_F11] = "f11",
	[KEY_FN_F12] = "f12",
	[KEY_FN_1] = "1",
	[KEY_FN_2] = "2",
	[KEY_FN_D] = "d",
	[KEY_FN_E] = "e",
	[KEY_FN_F] = "f",
	[KEY_FN_S] = "s",
	[KEY_FN_B] = "b",
	[KEY_BRL_DOT1] = "dot1",
	[KEY_BRL_DOT2] = "dot2",
	[KEY_BRL_DOT3] = "dot3",
	[KEY_BRL_DOT4] = "dot4",
	[KEY_BRL_DOT5] = "dot5",
	[KEY_BRL_DOT6] = "dot6",
	[KEY_BRL_DOT7] = "dot7",
	[KEY_BRL_DOT8] = "dot8",
	[KEY_BRL_DOT9] = "dot9",
	[KEY_BRL_DOT10] = "dot10",
	[KEY_NUMERIC_0] = "0",
	[KEY_NUMERIC_1] = "1",
	[KEY_NUMERIC_2] = "2",
	[KEY_NUMERIC_3] = "3",
	[KEY_NUMERIC_4] = "4",
	[KEY_NUMERIC_5] = "5",
	[KEY_NUMERIC_6] = "6",
	[KEY_NUMERIC_7] = "7",
	[KEY_NUMERIC_8] = "8",
	[KEY_NUMERIC_9] = "9",
	[KEY_NUMERIC_STAR] = "star",
	[KEY_NUMERIC_POUND] = "pound",
	[KEY_NUMERIC_A] = "a",
	[KEY_NUMERIC_B] = "b",
	[KEY_NUMERIC_C] = "c",
	[KEY_NUMERIC_D] = "d",
	[KEY_CAMERA_FOCUS] = "focus",
	[KEY_WPS_BUTTON] = "button",
	[KEY_TOUCHPAD_TOGGLE] = "toggle",
	[KEY_TOUCHPAD_ON] = "on",
	[KEY_TOUCHPAD_OFF] = "off",
	[KEY_CAMERA_ZOOMIN] = "zoomin",
	[KEY_CAMERA_ZOOMOUT] = "zoomout",
	[KEY_CAMERA_UP] = "up",
	[KEY_CAMERA_DOWN] = "down",
	[KEY_CAMERA_LEFT] = "left",
	[KEY_CAMERA_RIGHT] = "right",
	[KEY_ATTENDANT_ON] = "on",
	[KEY_ATTENDANT_OFF] = "off",
	[KEY_ATTENDANT_TOGGLE] = "toggle",
	[KEY_LIGHTS_TOGGLE] = "toggle",
	[KEY_ALS_TOGGLE] = "toggle",
	[KEY_BUTTONCONFIG] = "buttonconfig",
	[KEY_TASKMANAGER] = "taskmanager",
	[KEY_JOURNAL] = "journal",
	[KEY_CONTROLPANEL] = "controlpanel",
	[KEY_APPSELECT] = "appselect",
	[KEY_SCREENSAVER] = "screensaver",
	[KEY_VOICECOMMAND] = "voicecommand",
	[KEY_BRIGHTNESS_MIN] = "min",
	[KEY_BRIGHTNESS_MAX] = "max",
	[KEY_KBDINPUTASSIST_PREV] = "prev",
	[KEY_KBDINPUTASSIST_NEXT] = "next",
	[KEY_KBDINPUTASSIST_PREVGROUP] = "prevgroup",
	[KEY_KBDINPUTASSIST_NEXTGROUP] = "nextgroup",
	[KEY_KBDINPUTASSIST_ACCEPT] = "accept",
	[KEY_KBDINPUTASSIST_CANCEL] = "cancel",
	[KEY_RIGHT_UP] = "up",
	[KEY_RIGHT_DOWN] = "down",
	[KEY_LEFT_UP] = "up",
	[KEY_LEFT_DOWN] = "down",
	[KEY_ROOT_MENU] = "menu",
	[KEY_MEDIA_TOP_MENU] = "menu",
	[KEY_NUMERIC_11] = "11",
	[KEY_NUMERIC_12] = "12",
	[KEY_AUDIO_DESC] = "desc",
	[KEY_3D_MODE] = "mode",
	[KEY_NEXT_FAVORITE] = "favorite",
	[KEY_STOP_RECORD] = "record",
	[KEY_PAUSE_RECORD] = "record",
	[KEY_VOD] = "vod",
	[KEY_UNMUTE] = "unmute",
	[KEY_FASTREVERSE] = "fastreverse",
	[KEY_SLOWREVERSE] = "slowreverse",
	[KEY_DATA] = "data",
	[KEY_MAX] = "max"
};

#endif
